/** \file
 * Header for the definition of some Atl basic types
 *
 * \author Abvent R&D Copyright (c) 2009
 * \author Guillaume Boulet-Gercourt
 * \version 1.0
 * \date 17/11/2009
 */

#ifndef ATL_types_h
#define ATL_types_h

#include <cassert>
#include <limits>

// Last chance to undef min and max
#ifdef max
#undef max
#endif
#ifdef min
#undef min
#endif



namespace  AvAtl {

// forward declarations
class CAtlMatrix;


// Numeric limits
#define	ATL_NUMLIMIT_MAX(T)	std::numeric_limits<T>::max()
#define	ATL_NUMLIMIT_MIN(T)	std::numeric_limits<T>::min()


/** \class __Atl_void_ptr_to_scalar 
 *	\brief Conversion from ptr to scalar
 *
 * \author Abvent R&D Copyright (c) 2009
 * \author Guillaume Boulet-Gercourt
 * \date 17/11/2009
 */
template <typename Type, typename CheckPtrType>
union __Atl_void_ptr_to_scalar
{
	inline void	check_sizes()
	{
		assert(sizeof (mPtr) >= sizeof (mScalar));
		assert((CheckPtrType)mPtr <= (CheckPtrType)ATL_NUMLIMIT_MAX(Type));
	}
	
	__Atl_void_ptr_to_scalar(void* ptr) : mPtr(ptr) { check_sizes(); }
	__Atl_void_ptr_to_scalar(Type scalar) : mScalar(scalar) { check_sizes(); }

	
	void*	mPtr;
	Type	mScalar;
};

#define	ATL_INT_TO_PTR(Scalar)	(AvAtl::__Atl_void_ptr_to_scalar<int, intptr_t>(Scalar).mPtr)
#define	ATL_PTR_TO_INT(Ptr)		(AvAtl::__Atl_void_ptr_to_scalar<int, intptr_t>((void*)(Ptr)).mScalar)

#define	ATL_UINT_TO_PTR(Scalar)	(AvAtl::__Atl_void_ptr_to_scalar<unsigned int, uintptr_t>(Scalar).mPtr)
#define	ATL_PTR_TO_UINT(Ptr)	(AvAtl::__Atl_void_ptr_to_scalar<unsigned int, uintptr_t>((void*)(Ptr)).mScalar)

#define	ATL_LONG_TO_PTR(Scalar)	(AvAtl::__Atl_void_ptr_to_scalar<long, intptr_t>(Scalar).mPtr)
#define	ATL_PTR_TO_LONG(Ptr)		(AvAtl::__Atl_void_ptr_to_scalar<long, intptr_t>((void*)(Ptr)).mScalar)

#define	ATL_ULONG_TO_PTR(Scalar)	(AvAtl::__Atl_void_ptr_to_scalar<unsigned long, uintptr_t>(Scalar).mPtr)
#define	ATL_PTR_TO_ULONG(Ptr)	(AvAtl::__Atl_void_ptr_to_scalar<unsigned long, uintptr_t>((void*)(Ptr)).mScalar)

	
/** \class __AtlStrTypeFormat 
 *	\brief Misc
 *
 * \author Abvent R&D Copyright (c) 2009
 * \author Guillaume Boulet-Gercourt
 * \date 17/11/2009
 */
template <typename T>
class		__AtlStrTypeFormat
{
public:
	static const unsigned char	StrTypeFormat[];
};
	/// NE COMPILE PAS SOUS MAC. EN ATTENDANT DE COMPRENDRE POURQUOI JE CONTOURNE.
#if defined(_64) || defined(__LP64__)
	//template <> const unsigned char	__AtlStrTypeFormat<ATLReal>::StrTypeFormat[] = "%lf";
	//template <> const unsigned char	__AtlStrTypeFormat<CAtlMatrix>::StrTypeFormat[] = "%lf, %lf, %lf, %lf";
#define ATL_STRFORMAT_AtlMatrix(Tmp)	"%lf, %lf, %lf, %lf"
#define ATL_STRFORMAT_ATLCoord(Tmp)	"%lf"
	
#else
	//template <> const unsigned char	__AtlStrTypeFormat<ATLReal>::StrTypeFormat[] = "%lf";
	//template <> const unsigned char	__AtlStrTypeFormat<CAtlMatrix>::StrTypeFormat[] = "%lf, %lf, %lf, %lf";
	
#define ATL_STRFORMAT_AtlMatrix(Tmp)	"%lf, %lf, %lf, %lf"
#define ATL_STRFORMAT_ATLCoord(Tmp)	"%lf"

	
#endif

/// Create a __AtlStrTypeFormat instance
#define	ATL_STRFORMAT(Type)	(AvAtl::Utf8)(AvAtl::__AtlStrTypeFormat<Type>::StrTypeFormat)

} // namespace AvAtl

#endif ATL_types_h
