/** \file
 * Header for the CAtlVector class
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Richard Young
 * \version 1.0
 * \date 04/02/2002
 */

#ifndef CAtlVector_h
#define CAtlVector_h

#ifndef AtlTypes_h
	#include "AvAtlTypes.h"
#endif

#include <stdexcept>

namespace  AvAtl {


/// Description of the 3d vector.
/** \class CAtlVector
 * The class CAtlVector is a structure use to describe the 3d space vector.
 * \note Indexing will throw an exception if the index is invalid.
 *
 * \author Abvent R&D Copyright (c) 2002
 * \author Richard Young
 * \version 1.0
 * \date 04/02/2002
 */
class CAtlVector
{
public:	
	typedef	double	ATLCoord;

	/// Constructor.
					CAtlVector()
						:	mX(0), mY(0), mZ(0)
						{}
	/// Constructor.
					CAtlVector(ATLCoord x, ATLCoord y, ATLCoord z)
						:	mX(x), mY(y), mZ(z)
						{}

	/// Getter
	ATLCoord		X() const	{ return mX; }
   /// Setter
	void			X(ATLCoord x) { mX = x; }
   /// Getter
	ATLCoord		Y() const	{ return mY; }
   /// Setter
	void			Y(ATLCoord y) { mY = y; }
   /// Getter
	ATLCoord		Z() const	{ return mZ; }
   /// Setter
	void			Z(ATLCoord z) { mZ = z; }

	/// Return the point specified by the index.
	ATLCoord		operator[](unsigned int i) const { TestIndex(i);  return (&mX)[i]; }

	/// Return the point specified by the index.
	ATLCoord&		operator[](unsigned int i) { TestIndex(i);  return (&mX)[i]; }

private:
	/// Validate the index, throw if invalid
	static void	TestIndex(unsigned int i)
					{
						if (i > 2)
							throw std::out_of_range("CAtlVector::TestIndex");
					}

   /// Spacial X component.
	ATLCoord		mX;
   /// Spacial Y component.
	ATLCoord		mY;
   /// Spacial Z component.
	ATLCoord		mZ;
};

}

#endif
