/*!
	\file	IAtlInternalComponentInstance.h
	\author Abvent R&D Copyright 2004
	\author Guillaume Boulet-Gercourt
	\date 24/03/2009
	\brief Definition of IAtlInternalComponentInstance interface
*/
#ifndef		IAtlInternalComponentInstance_h
#define	IAtlInternalComponentInstance_h

#include "AvAtlTypes.h"
#include "CAtlProperty.h"
#include "TAtlObjectMgr.h"



namespace AvAtl {


// Forward declaration
class IAtlObjectInstance;

/// List of properties
typedef TAtlObjectMgr<CAtlProperty>			AtlPropertyMgr;

/*!
	\addtogroup Atl Objects
	\{
*/


/// Interface for atl internal component references
/** \interface IAtlInternalComponentInstance 
*	Interface that represent atl internal component reference
*
* \author Abvent R&D Copyright (c) 2009
* \author Guillaume Boulet-Gercourt
* \version 1.0
* \date 24/03/2009
*/
class IAtlInternalComponentInstance
{
public:	//! Get the attached heliodon name
	/// Type conversion to IAtlObjectInstance
	virtual const IAtlObjectInstance*	asObjectInstance() const = 0;

	/// Type conversion to IAtlObjectInstance
	virtual IAtlObjectInstance*			asObjectInstance() = 0;

	/// Get the internal component id
	virtual tUUID						getComponentId() const = 0;
	
	/// Set the internal component id
	virtual void						setComponentId(tUUID inId) = 0;

	/// Get the  properties manager of the internal component.
	/** For consulting */
	virtual const AtlPropertyMgr*	getPropertyMgr() const = 0;

	/// Get the  properties manager of the internal component.
	/** For setting */
	virtual AtlPropertyMgr*			getPropertyMgr() = 0;

protected:
	/** You must not create or delete objects of this class.
	    This is the responsibility of the CAtlMgr class. */
	virtual							~IAtlInternalComponentInstance() {}
private:
};

/*!
	\} // group Atl Objects
*/

}		// namespace AvAtl

#endif		// IAtlObjectRef_h
