/** \file
 * Header for the IAtlPanorama2 interface
 *
 * \author Abvent R&D Copyright (c) 2011
 * \author Christophe
 * \date 18/02/2011
 */

#ifndef IAtlPanoramaV2_h
#define IAtlPanoramaV2_h


// Basic class.
#ifndef AtlTypes_h
	#include "AvAtlTypes.h"
#endif
#ifndef TAtlObjectMgr_h
	#include "TAtlObjectMgr.h"
#endif
#ifndef IAtlPanoNodeV2_h
	#include "IAtlPanoNodeV2.h"
#endif


// We use this namespace for our library.
namespace  AvAtl {


// List of panoramas
typedef TAtlObjectMgr<class IAtlPanoramaV2>	AtlPanoramaV2Mgr;
	
	
// Manager of instances
typedef TAtlObjectMgr<class IAtl3DPath>		Atl3DPathsMgr;


/// Interface for panorama2
/// This is the new version of the multi-node panorama using the PromenAdd technology
/** \interface IAtlPanorama 2
 *	Interface for panorama2
 *
 * \author Abvent R&D Copyright (c) 2011
 * \author Christophe
 * \date 18/02/2011
 */
class IAtlPanoramaV2
{
protected:
	/** You must not create or delete objects of this class.
	    This is the responsibility of the CAtlMgr class. */
	virtual								~IAtlPanoramaV2() {}

public:
	/// Put an XML representation of the group inside the father node.
	/** \retval the new node of the ambient.
		\retval NULL if an error occurs. */
	virtual XMLNodeRef					putInXML(XMLNodeRef inFatherNode) const = 0;
	/// Initialize the group with the XML.
	/** \retval true intialization from XML succeed.
		\retval false an error occurs. */
	virtual bool						initWithXML(XMLNodeRef inFatherNode, bool inCopyUUId = false) = 0;

	/// Return the name.
	/** Do not delete this string. */
	virtual tUtf8						getName() const = 0;

	/// Set the name.
	/** \param inName is in unicode. */
	virtual void						setName(tUtf8 inName) = 0;

	/// Return the link manager.
	/** For consulting. */
	virtual const AtlPanoLinkV2Mgr*		getLinkMgr() const = 0;
	/// Return the links of this panorama.
	/** For settings. */
	virtual AtlPanoLinkV2Mgr*			getLinkMgr() = 0;
	
	/// Return the node manager.
	/** For consulting. */
	virtual const AtlPanoNodeV2Mgr*		getNodeMgr() const = 0;
	/// Return the nodes of this panorama.
	/** For settings. */
	virtual AtlPanoNodeV2Mgr*			getNodeMgr() = 0;

#if 0 // TODO: Move panolink's 3DPaths here. Seems to be the good place to be, but Artlantis use the CATLMgr::get3DPathsMgr
	/// Return the 3D path manager.
	/** For consulting. */
	virtual const Atl3DPathsMgr*		Xget3DPathMgr() const = 0;
	/// Return the links of this panorama.
	/** For settings. */
	virtual Atl3DPathsMgr*				Xget3DPathMgr() = 0;
#endif

	/// Return the first node of this panorama.
	virtual tUUID						getFirstNode() const = 0;
	/// Set the first node of this panorama.
	virtual void						setFirstNode(tUUID inCurrent) = 0;

	/// Return the initial node direction of this panorama
	virtual const CAtlVector&			getInitialDirection() const = 0;
	virtual const CAtlVector&			getPreviewDirection() const = 0;
	/// Set the initial node direction of this panorama
	virtual void						setInitialDirection(const CAtlVector& inDirection) = 0;
	virtual void						setPreviewDirection(const CAtlVector& inDirection) = 0;

	/// Return the initial node focal of this panorama
	virtual double						getInitialFocal() const = 0;
	virtual double						getPreviewFocal() const = 0;
	/// Set the initial node focal of this panorama
	virtual void						setInitialFocal(double inFocal) = 0;
	virtual void						setPreviewFocal(double inFocal) = 0;
	
	/// Return the options of the view.
	/** For consulting. */
	virtual const class IAtlViewOpt*	getViewOpt() const = 0;
	/// Return the options of the view.
	/** For settings. */
	virtual class IAtlViewOpt*			getViewOpt() = 0;

	/// Return the uuid of the panorama
	virtual tUUID						getId() const = 0;
	/// Set the uuid of the panorama
	virtual void						setId(tUUID inId) = 0;

	/// Return the uuid of the panorama
	virtual tUUID						getCurrentNodeId() const = 0;
	/// Set the uuid of the panorama
	virtual void						setCurrentNodeId(tUUID inId) = 0;

	/// Return the view info of this view.
	virtual const class IAtlViewInfo*	getViewInfo() const = 0;
	/// Return the view info of this view.
	virtual class IAtlViewInfo*			getViewInfo() = 0;

	/// Get additional value. for internal usage
	virtual int							getUIValue() const = 0;
	/// Set additional value. for internal usage only
	virtual void						setUIValue(int inValue) = 0;

	/// Get additional value. for internal usage
	virtual int							getViewFlagsValue() const = 0;
	/// Set additional value. for internal usage only
	virtual void						setViewFlagsValue(int inValue) = 0;

    virtual bool                        getSwitchToInitialDirection() const = 0;
    virtual void                        setSwitchToInitialDirection( bool value ) = 0;
};


} // namespace  AvAtl

#endif
